import { a as push, s as store_get, u as unsubscribe_stores, c as bind_props, p as pop } from "./index2.js";
import { p as page } from "./stores.js";
import { g as getBestThumbnail } from "./images.js";
import "human-number";
import { Capacitor } from "@capacitor/core";
import "@capacitor/screen-orientation";
import "@capacitor/status-bar";
import "@hugotomazi/capacitor-navigation-bar";
import { AudioPlayer } from "@mediagrid/capacitor-native-audio";
import "sponsorblock-api";
import { o as onDestroy } from "./index-server.js";
import "./runtime.js";
import { g as get } from "./index.js";
import { s as saveVideoProgress, o as deleteVideoProgress } from "./index3.js";
import "./client.js";
import "he";
import { o as silenceSkipperStore, q as playerAutoPlayStore, t as playerAlwaysLoopStore, p as playerProxyVideosStore, d as playerSavePlaybackPositionStore, u as synciousStore, n as synciousInstanceStore, b as authStore, f as instanceStore } from "./store.js";
import "./beer.min.js";
import { a as attr } from "./attributes.js";
import { e as escape_html } from "./escaping.js";
function Player($$payload, $$props) {
  push();
  var $$store_subs;
  let {
    data,
    audioMode = false,
    player = void 0,
    isSyncing = false,
    isEmbed = false,
    segments = []
  } = $$props;
  let src = [];
  let snackBarAlert = "";
  let userVideoSpeed = 1;
  let silenceIsFastForwarding = false;
  let silenceSkipperInterval;
  function loadTimeFromUrl(page2) {
    if (player) {
      const timeGivenUrl = page2.url.searchParams.get("time");
      if (timeGivenUrl && !isNaN(parseFloat(timeGivenUrl))) {
        player.currentTime = Number(timeGivenUrl);
        return true;
      }
    }
    return false;
  }
  page.subscribe((pageUpdate) => loadTimeFromUrl(pageUpdate));
  let initStoreSilence = true;
  silenceSkipperStore.subscribe((value) => {
    if (initStoreSilence) {
      initStoreSilence = false;
      return;
    }
    if (typeof silenceSkipperInterval !== "undefined") {
      clearInterval(silenceSkipperInterval);
    }
    if (value) {
      initSilenceSkipper();
    }
  });
  get(playerProxyVideosStore);
  function fastForwardToSound(videoElement, threshold = 0.01, checkInterval = 100, fastForwardRate = 4) {
    if (!(videoElement instanceof HTMLMediaElement)) {
      console.error("Provided element is not a valid HTMLMediaElement");
      return;
    }
    const audioContext = new AudioContext();
    const source = audioContext.createMediaElementSource(videoElement);
    const analyser = audioContext.createAnalyser();
    source.connect(analyser);
    analyser.connect(audioContext.destination);
    analyser.fftSize = 256;
    const dataArray = new Uint8Array(analyser.frequencyBinCount);
    const checkForSound = () => {
      analyser.getByteFrequencyData(dataArray);
      const averageVolume = dataArray.reduce((a, b) => a + b, 0) / dataArray.length;
      const volume = averageVolume / 255;
      if (volume < threshold) {
        if (!silenceIsFastForwarding) {
          videoElement.playbackRate = fastForwardRate;
          silenceIsFastForwarding = true;
        }
      } else {
        if (silenceIsFastForwarding) {
          videoElement.playbackRate = userVideoSpeed;
          silenceIsFastForwarding = false;
        }
      }
    };
    silenceSkipperInterval = setInterval(checkForSound, checkInterval);
  }
  function initSilenceSkipper() {
    const videoContainer = document.getElementById("video");
    const videoElement = videoContainer.querySelector("video");
    fastForwardToSound(videoElement);
  }
  function savePlayerPos() {
    if (data.video.hlsUrl) return;
    if (get(playerSavePlaybackPositionStore) && player && player.currentTime) {
      if (player.currentTime < player.duration - 10 && player.currentTime > 10) {
        try {
          localStorage.setItem(`v_${data.video.videoId}`, player.currentTime.toString());
        } catch {
        }
        if (get(synciousStore) && get(synciousInstanceStore) && get(authStore)) {
          saveVideoProgress(data.video.videoId, player.currentTime);
        }
      } else {
        try {
          localStorage.removeItem(`v_${data.video.videoId}`);
        } catch {
        }
        if (get(synciousStore) && get(synciousInstanceStore) && get(authStore)) {
          deleteVideoProgress(data.video.videoId);
        }
      }
    }
  }
  onDestroy(async () => {
    if (Capacitor.getPlatform() === "android") {
      await AudioPlayer.destroy({ audioId: data.video.videoId });
    }
    if (typeof silenceSkipperInterval !== "undefined") {
      clearInterval(silenceSkipperInterval);
    }
    try {
      savePlayerPos();
    } catch (error) {
    }
    await player.pause();
    player.destroy();
  });
  if (audioMode) {
    $$payload.out += "<!--[-->";
    $$payload.out += `<div style="margin-top: 40vh;"></div>`;
  } else {
    $$payload.out += "<!--[!-->";
  }
  $$payload.out += `<!--]--> <media-player id="video"${attr("autoplay", store_get($$store_subs ??= {}, "$playerAutoPlayStore", playerAutoPlayStore) && !isSyncing, true)}${attr("loop", store_get($$store_subs ??= {}, "$playerAlwaysLoopStore", playerAlwaysLoopStore), true)}${attr("title", data.video.title)}${attr("streamtype", "on-demand")}${attr("viewtype", audioMode ? "audio" : "video")} keep-alive=""${attr("src", src)}><media-provider>`;
  if (!audioMode) {
    $$payload.out += "<!--[-->";
    $$payload.out += `<media-poster class="vds-poster"${attr("src", getBestThumbnail(data.video.videoThumbnails, 1251, 781))}></media-poster>`;
  } else {
    $$payload.out += "<!--[!-->";
  }
  $$payload.out += `<!--]--></media-provider> <media-audio-layout></media-audio-layout> `;
  if (data.video.storyboards && data.video.storyboards.length > 3) {
    $$payload.out += "<!--[-->";
    $$payload.out += `<media-video-layout${attr("thumbnails", `${get(instanceStore)}${data.video.storyboards[3].url}`)}></media-video-layout>`;
  } else {
    $$payload.out += "<!--[!-->";
    $$payload.out += `<media-video-layout></media-video-layout>`;
  }
  $$payload.out += `<!--]--></media-player> `;
  if (!isEmbed) {
    $$payload.out += "<!--[-->";
    $$payload.out += `<div class="snackbar" id="snackbar-alert"><span class="bold" style="text-transform: capitalize;">${escape_html(snackBarAlert)}</span></div>`;
  } else {
    $$payload.out += "<!--[!-->";
  }
  $$payload.out += `<!--]-->`;
  if ($$store_subs) unsubscribe_stores($$store_subs);
  bind_props($$props, { player, segments });
  pop();
}
export {
  Player as P
};
