import { j as poTokenCacheStore, k as interfaceRegionStore, f as instanceStore, b as authStore, l as playerYouTubeJsAlways, m as playerYouTubeJsFallback, h as returnYTDislikesInstanceStore, n as synciousInstanceStore } from "./store.js";
import { n as numberWithCommas } from "./time.js";
import { CapacitorHttp, Capacitor } from "@capacitor/core";
import { BG } from "bgutils-js";
import { g as get } from "./index.js";
import { ProtoUtils, Utils, Innertube, UniversalCache } from "youtubei.js";
const CAPACITOR_HTTP_INTERCEPTOR = "/_capacitor_http_interceptor_";
const CAPACITOR_HTTP_INTERCEPTOR_URL_PARAM = "u";
const isRelativeOrProxyUrl = (url) => !url || !(url.startsWith("http:") || url.startsWith("https:")) || url.indexOf(CAPACITOR_HTTP_INTERCEPTOR) > -1;
const createProxyUrl = (url, win) => {
  if (isRelativeOrProxyUrl(url)) return url;
  const bridgeUrl = new URL(win.Capacitor?.getServerUrl() ?? "");
  bridgeUrl.pathname = CAPACITOR_HTTP_INTERCEPTOR;
  bridgeUrl.searchParams.append(CAPACITOR_HTTP_INTERCEPTOR_URL_PARAM, url);
  return bridgeUrl.toString();
};
const readFileAsBase64 = (file) => new Promise((resolve, reject) => {
  const reader = new FileReader();
  reader.onloadend = () => {
    const data = reader.result;
    resolve(btoa(data));
  };
  reader.onerror = reject;
  reader.readAsBinaryString(file);
});
const convertFormData = async (formData) => {
  const newFormData = [];
  for (const pair of formData.entries()) {
    const [key, value] = pair;
    if (value instanceof File) {
      const base64File = await readFileAsBase64(value);
      newFormData.push({
        key,
        value: base64File,
        type: "base64File",
        contentType: value.type,
        fileName: value.name
      });
    } else {
      newFormData.push({ key, value, type: "string" });
    }
  }
  return newFormData;
};
const convertBody = async (body, contentType) => {
  if (body instanceof ReadableStream || body instanceof Uint8Array) {
    let encodedData;
    if (body instanceof ReadableStream) {
      const reader = body.getReader();
      const chunks = [];
      while (true) {
        const { done, value } = await reader.read();
        if (done) break;
        chunks.push(value);
      }
      const concatenated = new Uint8Array(
        chunks.reduce((acc, chunk) => acc + chunk.length, 0)
      );
      let position = 0;
      for (const chunk of chunks) {
        concatenated.set(chunk, position);
        position += chunk.length;
      }
      encodedData = concatenated;
    } else {
      encodedData = body;
    }
    let data = new TextDecoder().decode(encodedData);
    let type;
    if (contentType === "application/json") {
      try {
        data = JSON.parse(data);
      } catch (ignored) {
      }
      type = "json";
    } else if (contentType === "multipart/form-data") {
      type = "formData";
    } else if (contentType?.startsWith("image")) {
      type = "image";
    } else if (contentType === "application/octet-stream") {
      type = "binary";
    } else {
      type = "text";
    }
    return {
      data,
      type,
      headers: { "Content-Type": contentType || "application/octet-stream" }
    };
  } else if (body instanceof URLSearchParams) {
    return {
      data: body.toString(),
      type: "text"
    };
  } else if (body instanceof FormData) {
    const formData = await convertFormData(body);
    return {
      data: formData,
      type: "formData"
    };
  } else if (body instanceof File) {
    const fileData = await readFileAsBase64(body);
    return {
      data: fileData,
      type: "file",
      headers: { "Content-Type": body.type }
    };
  }
  return { data: body, type: "json" };
};
const capacitorFetch = async (resource, options) => {
  const request = new Request(resource, options);
  const { method } = request;
  if (method.toLocaleUpperCase() === "GET" || method.toLocaleUpperCase() === "HEAD" || method.toLocaleUpperCase() === "OPTIONS" || method.toLocaleUpperCase() === "TRACE") {
    if (typeof resource === "string") {
      return await fetch(
        createProxyUrl(resource, window),
        options
      );
    } else if (resource instanceof Request) {
      const modifiedRequest = new Request(
        createProxyUrl(resource.url, window),
        resource
      );
      return await fetch(modifiedRequest, options);
    }
  }
  const tag = `CapacitorHttp fetch ${Date.now()} ${resource}`;
  console.time(tag);
  try {
    const { body } = request;
    const optionHeaders = Object.fromEntries(request.headers.entries());
    const {
      data: requestData,
      type,
      headers
    } = await convertBody(
      options?.body || body || void 0,
      optionHeaders["Content-Type"] || optionHeaders["content-type"]
    );
    const nativeResponse = await CapacitorHttp.request({
      url: request.url,
      method,
      data: requestData,
      dataType: type,
      headers: {
        ...headers,
        ...optionHeaders
      }
    });
    const contentType = nativeResponse.headers["Content-Type"] || nativeResponse.headers["content-type"];
    let data = contentType?.startsWith("application/json") ? JSON.stringify(nativeResponse.data) : nativeResponse.data;
    if (nativeResponse.status === 204) {
      data = null;
    }
    const response = new Response(data, {
      headers: nativeResponse.headers,
      status: nativeResponse.status
    });
    Object.defineProperty(response, "url", {
      value: nativeResponse.url
    });
    console.timeEnd(tag);
    return response;
  } catch (error) {
    console.timeEnd(tag);
    return Promise.reject(error);
  }
};
async function getPo(identifier) {
  const requestKey = "O43z0dpjhgX20SCx4KAo";
  const bgConfig = {
    fetch: Capacitor.getPlatform() === "android" ? capacitorFetch : fetch,
    globalObj: window,
    requestKey,
    identifier
  };
  const bgChallenge = await BG.Challenge.create(bgConfig);
  if (!bgChallenge)
    throw new Error("Could not get challenge");
  const interpreterJavascript = bgChallenge.interpreterJavascript.privateDoNotAccessOrElseSafeScriptWrappedValue;
  if (interpreterJavascript) {
    new Function(interpreterJavascript)();
  } else throw new Error("Could not load VM");
  const poTokenResult = await BG.PoToken.generate({
    program: bgChallenge.program,
    globalName: bgChallenge.globalName,
    bgConfig
  });
  return poTokenResult.poToken;
}
async function patchYoutubeJs(videoId) {
  let tokens;
  if (!Capacitor.isNativePlatform()) {
    throw new Error("Platform not supported");
  }
  const poTokenCache = get(poTokenCacheStore);
  if (!poTokenCache) {
    const visitorData = ProtoUtils.encodeVisitorData(Utils.generateRandomString(11), Math.floor(Date.now() / 1e3));
    const poToken = await getPo(visitorData);
    if (!poToken) {
      throw new Error("Unable to generate PO token");
    }
    tokens = {
      visitor_data: visitorData,
      po_token: poToken
    };
    poTokenCacheStore.set(tokens);
  } else {
    tokens = poTokenCache;
  }
  const youtube = await Innertube.create({
    visitor_data: tokens.visitor_data,
    po_token: tokens.po_token,
    fetch: Capacitor.getPlatform() === "android" ? capacitorFetch : fetch,
    generate_session_locally: true,
    cache: new UniversalCache(false),
    location: get(interfaceRegionStore)
  });
  const video = await youtube.getInfo(videoId);
  if (!video.primary_info || !video.secondary_info) {
    throw new Error("Unable to pull video info from youtube.js");
  }
  let dashUri = "";
  if (!video.basic_info.is_live) {
    let manifest = await video.toDash();
    manifest = manifest.replaceAll("<EssentialProperty", "<SupplementalProperty");
    dashUri = URL.createObjectURL(new Blob([manifest], { type: "application/dash+xml;charset=utf8" }));
  }
  const descString = video.secondary_info.description?.toString() || "";
  let authorThumbnails;
  if (video.basic_info.channel_id) {
    const channel = await youtube.getChannel(video.basic_info.channel_id);
    authorThumbnails = channel.metadata.avatar;
  } else {
    authorThumbnails = [];
  }
  let recommendedVideos = [];
  video.watch_next_feed?.forEach((recommended) => {
    recommendedVideos.push({
      videoThumbnails: recommended?.thumbnails || [],
      videoId: recommended?.id || "",
      title: recommended?.title?.toString() || "",
      viewCountText: recommended.view_count ? numberWithCommas(Number(recommended?.view_count.toString().replace(/\D/g, ""))) || "" : "",
      lengthSeconds: recommended?.duration?.seconds || 0,
      author: recommended?.author?.name || "",
      authorId: recommended?.author?.id || ""
    });
  });
  let captions = [];
  video.captions?.caption_tracks?.forEach((caption) => {
    captions.push({
      label: caption.name?.toString() || "",
      language_code: caption.language_code,
      // Add correct format to url.
      url: caption.base_url + "&fmt=vtt"
    });
  });
  let adaptiveFormats = [];
  video.streaming_data?.adaptive_formats.forEach((format) => {
    adaptiveFormats.push({
      index: format.index_range?.start?.toString() || "",
      bitrate: format.bitrate?.toString() || "",
      init: format.init_range?.start?.toString() || "",
      url: format.url || "",
      itag: format.itag?.toString() || "",
      type: format.mime_type,
      clen: "",
      lmt: "",
      projectionType: 0,
      resolution: format.width ? `${format.width}x${format.height}` : void 0
    });
  });
  let storyboard = [];
  if (video.storyboards && "boards" in video.storyboards) {
    video.storyboards.boards.forEach((board) => {
      storyboard.push({
        templateUrl: board.template_url,
        url: board.template_url,
        count: board.storyboard_count,
        height: board.thumbnail_height,
        width: board.thumbnail_width,
        interval: board.interval,
        storyboardCount: board.storyboard_count,
        storyboardHeight: board.thumbnail_height,
        storyboardWidth: board.thumbnail_width
      });
    });
  }
  console.log(video.primary_info);
  return {
    type: "video",
    title: video.primary_info.title?.toString() || "",
    viewCount: Number(video.primary_info.view_count?.original_view_count || 0),
    viewCountText: video.primary_info.view_count?.original_view_count.toString() || "0",
    likeCount: video.basic_info.like_count || 0,
    dislikeCount: 0,
    allowRatings: false,
    rating: 0,
    isListed: 0,
    isFamilyFriendly: video.basic_info.is_family_safe || true,
    genre: video.basic_info.category || "",
    genreUrl: "",
    dashUrl: dashUri,
    adaptiveFormats,
    formatStreams: [],
    recommendedVideos,
    authorThumbnails,
    captions,
    authorId: video.basic_info.channel_id || "",
    authorUrl: `/channel/${video.basic_info.channel_id}`,
    authorVerified: false,
    description: descString,
    descriptionHtml: video.secondary_info.description?.toHTML() || descString,
    published: 0,
    publishedText: video.primary_info.published?.toString() || "",
    premiereTimestamp: 0,
    hlsUrl: video.streaming_data?.hls_manifest_url || void 0,
    liveNow: video.basic_info.is_live || false,
    premium: false,
    storyboards: storyboard,
    isUpcoming: false,
    videoId,
    videoThumbnails: video.basic_info.thumbnail,
    author: video.basic_info.author || "Unknown",
    lengthSeconds: video.basic_info.duration || 0,
    subCountText: "",
    keywords: video.basic_info.keywords || [],
    allowedRegions: [],
    fallbackPatch: "youtubejs"
  };
}
function buildPath(path) {
  return new URL(`${get(instanceStore)}/api/v1/${path}`);
}
function setRegion(url) {
  const region = get(interfaceRegionStore);
  if (region) {
    url.searchParams.set("region", region);
  }
  return url;
}
async function fetchErrorHandle(response) {
  if (!response.ok) {
    let message = "Internal error";
    try {
      const json = await response.json();
      message = "errorBacktrace" in json ? json.errorBacktrace : json.error;
    } catch {
    }
    throw Error(message);
  }
  return response;
}
function buildAuthHeaders() {
  return { headers: { Authorization: `Bearer ${get(authStore)?.token}` } };
}
async function getTrending(fetchOptions) {
  const resp = await fetchErrorHandle(await fetch(setRegion(buildPath("trending")), fetchOptions));
  return await resp.json();
}
async function getPopular(fetchOptions) {
  const resp = await fetchErrorHandle(await fetch(buildPath("popular"), fetchOptions));
  return await resp.json();
}
async function getResolveUrl(url) {
  const resp = await fetchErrorHandle(await fetch(`${buildPath("resolveurl")}?url=${url}`));
  return await resp.json();
}
async function getVideo(videoId, local = false, fetchOptions) {
  const resp = await fetch(setRegion(buildPath(`videos/${videoId}?local=${local}`)), fetchOptions);
  if (get(playerYouTubeJsAlways) && Capacitor.isNativePlatform() || !resp.ok && get(playerYouTubeJsFallback) && Capacitor.isNativePlatform()) {
    return await patchYoutubeJs(videoId);
  } else {
    await fetchErrorHandle(resp);
  }
  return await resp.json();
}
async function getDislikes(videoId, fetchOptions) {
  const resp = await fetchErrorHandle(
    await fetch(`${get(returnYTDislikesInstanceStore)}/votes?videoId=${videoId}`, fetchOptions)
  );
  return await resp.json();
}
async function getComments(videoId, parameters, fetchOptions) {
  const path = buildPath(`comments/${videoId}`);
  path.search = new URLSearchParams(parameters).toString();
  const resp = await fetchErrorHandle(await fetch(path, fetchOptions));
  return await resp.json();
}
async function getChannel(channelId, fetchOptions) {
  const resp = await fetchErrorHandle(await fetch(buildPath(`channels/${channelId}`), fetchOptions));
  return await resp.json();
}
async function getHashtag(tag, page = 0) {
  const resp = await fetchErrorHandle(await fetch(buildPath(`hashtag/${tag}?page=${page}`)));
  return await resp.json();
}
async function getSearch(search, options, fetchOptions) {
  if (typeof options.sort_by === "undefined") {
    options.sort_by = "relevance";
  }
  if (typeof options.type === "undefined") {
    options.type = "video";
  }
  if (typeof options.page === "undefined") {
    options.page = "1";
  }
  const path = buildPath("search");
  path.search = new URLSearchParams({ ...options, q: search }).toString();
  const resp = await fetchErrorHandle(await fetch(setRegion(path), fetchOptions));
  return await resp.json();
}
async function getFeed(maxResults, page, fetchOptions = {}) {
  const path = buildPath("auth/feed");
  path.search = new URLSearchParams({
    max_results: maxResults.toString(),
    page: page.toString()
  }).toString();
  const resp = await fetchErrorHandle(await fetch(path, { ...buildAuthHeaders(), ...fetchOptions }));
  return await resp.json();
}
async function getSubscriptions(fetchOptions = {}) {
  const resp = await fetchErrorHandle(
    await fetch(buildPath("auth/subscriptions"), { ...buildAuthHeaders(), ...fetchOptions })
  );
  return await resp.json();
}
async function amSubscribed(authorId, fetchOptions = {}) {
  if (!get(authStore)) return false;
  try {
    const subscriptions = (await getSubscriptions(fetchOptions)).filter((sub) => sub.authorId === authorId);
    return subscriptions.length === 1;
  } catch {
    return false;
  }
}
async function postHistory(videoId, fetchOptions = {}) {
  await fetchErrorHandle(
    await fetch(buildPath(`auth/history/${videoId}`), {
      method: "POST",
      ...buildAuthHeaders(),
      ...fetchOptions
    })
  );
}
async function getPlaylist(playlistId, page = 1, fetchOptions = {}) {
  let resp;
  if (get(authStore)) {
    resp = await fetch(buildPath(`auth/playlists/${playlistId}?page=${page}`), { ...buildAuthHeaders(), ...fetchOptions });
  } else {
    resp = await fetch(buildPath(`playlists/${playlistId}?page=${page}`), fetchOptions);
  }
  await fetchErrorHandle(resp);
  return await resp.json();
}
async function getPersonalPlaylists(fetchOptions = {}) {
  const resp = await fetchErrorHandle(await fetch(buildPath("auth/playlists"), { ...buildAuthHeaders(), ...fetchOptions }));
  return await resp.json();
}
async function saveVideoProgress(videoId, time, fetchOptions = {}) {
  const headers = buildAuthHeaders();
  headers["headers"]["Content-type"] = "application/json";
  await fetchErrorHandle(
    await fetch(`${get(synciousInstanceStore)}/video/${encodeURIComponent(videoId)}`, {
      ...headers,
      method: "POST",
      body: JSON.stringify({
        time
      }),
      ...fetchOptions
    })
  );
}
async function deleteVideoProgress(videoId, fetchOptions = {}) {
  await fetchErrorHandle(
    await fetch(`${get(synciousInstanceStore)}/video/${videoId}`, {
      method: "DELETE",
      ...buildAuthHeaders(),
      ...fetchOptions
    })
  );
}
export {
  getChannel as a,
  getHashtag as b,
  getPlaylist as c,
  getPersonalPlaylists as d,
  getSearch as e,
  getFeed as f,
  getPopular as g,
  getSubscriptions as h,
  getTrending as i,
  getVideo as j,
  getResolveUrl as k,
  getComments as l,
  getDislikes as m,
  amSubscribed as n,
  deleteVideoProgress as o,
  postHistory as p,
  saveVideoProgress as s
};
