import { Capacitor } from "@capacitor/core";
import { w as writable } from "./index.js";
var stores = {
  local: {},
  session: {}
};
function getStorage(type) {
  return type === "local" ? localStorage : sessionStorage;
}
function persisted(key, initialValue, options) {
  var _a, _b, _c, _d, _e, _f, _g, _h;
  const serializer = (_a = void 0) != null ? _a : JSON;
  const storageType = (_b = void 0) != null ? _b : "local";
  const syncTabs = (_c = void 0) != null ? _c : true;
  const onWriteError = (_e = (_d = void 0) != null ? _d : void 0) != null ? _e : (e) => console.error(`Error when writing value from persisted store "${key}" to ${storageType}`, e);
  const onParseError = (_f = void 0) != null ? _f : (newVal, e) => console.error(`Error when parsing ${newVal ? '"' + newVal + '"' : "value"} from persisted store "${key}"`, e);
  const beforeRead = (_g = void 0) != null ? _g : (val) => val;
  const beforeWrite = (_h = void 0) != null ? _h : (val) => val;
  const browser = typeof window !== "undefined" && typeof document !== "undefined";
  const storage = browser ? getStorage(storageType) : null;
  function updateStorage(key2, value) {
    const newVal = beforeWrite(value);
    try {
      storage == null ? void 0 : storage.setItem(key2, serializer.stringify(newVal));
    } catch (e) {
      onWriteError(e);
    }
  }
  function maybeLoadInitial() {
    function serialize(json2) {
      try {
        return serializer.parse(json2);
      } catch (e) {
        onParseError(json2, e);
      }
    }
    const json = storage == null ? void 0 : storage.getItem(key);
    if (json == null) return initialValue;
    const serialized = serialize(json);
    if (serialized == null) return initialValue;
    const newVal = beforeRead(serialized);
    return newVal;
  }
  if (!stores[storageType][key]) {
    const initial = maybeLoadInitial();
    const store = writable(initial, (set2) => {
      if (browser && storageType == "local" && syncTabs) {
        const handleStorage = (event) => {
          if (event.key === key && event.newValue) {
            let newVal;
            try {
              newVal = serializer.parse(event.newValue);
            } catch (e) {
              onParseError(event.newValue, e);
              return;
            }
            const processedVal = beforeRead(newVal);
            set2(processedVal);
          }
        };
        window.addEventListener("storage", handleStorage);
        return () => window.removeEventListener("storage", handleStorage);
      }
    });
    const { subscribe, set } = store;
    stores[storageType][key] = {
      set(value) {
        set(value);
        updateStorage(key, value);
      },
      update(callback) {
        return store.update((last) => {
          const value = callback(last);
          updateStorage(key, value);
          return value;
        });
      },
      reset() {
        this.set(initialValue);
      },
      subscribe
    };
  }
  return stores[storageType][key];
}
function platformDependentDefault(givenValue, defaultValue) {
  if (defaultValue && Capacitor.getPlatform() !== "web") {
    return defaultValue;
  }
}
const instanceStore = persisted(
  "invidiousInstance",
  platformDependentDefault(void 0, "https://invidious.materialio.us")
);
const authStore = persisted(
  "authToken",
  null
);
const darkModeStore = persisted("darkMode", null);
const themeColorStore = persisted("themeColor", null);
const showWarningStore = persisted("showWarning", true);
const activePageStore = writable("home");
const playerAutoPlayStore = persisted("autoPlay", true);
const playerAlwaysLoopStore = persisted("alwaysLoop", false);
const playerProxyVideosStore = persisted("proxyVideos", true);
const playerListenByDefaultStore = persisted("listenByDefault", false);
const playerSavePlaybackPositionStore = persisted("savePlaybackPosition", true);
const playerTheatreModeByDefaultStore = persisted("theatreModeByDefault", false);
const playerAutoplayNextByDefaultStore = persisted("autoplayNextByDefault", false);
const playerMiniPlayerStore = persisted("miniPlayer", true);
const playerYouTubeJsFallback = persisted("youTubeJsFallback", true);
const playerYouTubeJsAlways = persisted("youTubeJsAlways", false);
const playerAndroidBgPlayer = persisted("androidBgPlayer", true);
const playerAndroidLockOrientation = persisted("androidLockOrientation", true);
const playerDefaultLanguage = persisted("defaultLanguage", "");
const returnYtDislikesStore = persisted("returnYtDislikes", false);
const returnYTDislikesInstanceStore = persisted(
  "returnYTDislikesInstance",
  platformDependentDefault(void 0, "https://ryd-proxy.materialio.us")
);
const synciousStore = persisted("syncious", true);
const synciousInstanceStore = persisted(
  "synciousInstance",
  platformDependentDefault(void 0, "https://extended-api.materialio.us")
);
const interfaceRegionStore = persisted("interfaceRegion", "US");
const interfaceSearchSuggestionsStore = persisted("searchSuggestions", true);
const interfacePreviewVideoOnHoverStore = persisted("previewVideoOnHover", true);
const interfaceForceCase = persisted("forceCase", null);
const interfaceAutoExpandComments = persisted("autoExpandComments", true);
const interfaceAutoExpandDesc = persisted("autoExpandDesc", false);
const interfaceAmoledTheme = persisted("amoledTheme", false);
const interfaceLowBandwidthMode = persisted("lowBandwidthMode", false);
const interfaceDisplayThumbnailAvatars = persisted("disableThumbnailAvatars", false);
const interfaceDefaultPage = persisted("defaultPage", "/");
const interfaceSearchHistoryEnabled = persisted("searchHistoryEnabled", false);
const sponsorBlockStore = persisted("sponsorBlock", true);
const sponsorBlockUrlStore = persisted(
  "sponsorBlockUrl",
  "https://sponsor.ajay.app"
);
const sponsorBlockCategoriesStore = persisted(
  "sponsorBlockCategories",
  []
);
const sponsorBlockDisplayToastStore = persisted(
  "sponsorBlockDisplayToast",
  false
);
const sponsorBlockTimelineStore = persisted(
  "sponsorBlockTimeline",
  false
);
const deArrowInstanceStore = persisted(
  "deArrowInstance",
  "https://sponsor.ajay.app"
);
const deArrowEnabledStore = persisted("deArrowEnabled", false);
const deArrowTitlesOnly = persisted("deArrowTitlesOnly", true);
const deArrowThumbnailInstanceStore = persisted(
  "deArrowThumbnailInstance",
  "https://dearrow-thumb.ajay.app"
);
const syncPartyPeerStore = writable(null);
const syncPartyConnectionsStore = writable();
const playlistSettingsStore = writable({});
const miniPlayerSrcStore = writable(null);
const silenceSkipperStore = persisted("silenceSkipper", false);
const poTokenCacheStore = writable();
persisted("searchHistory", []);
export {
  playerTheatreModeByDefaultStore as A,
  playerListenByDefaultStore as B,
  playerMiniPlayerStore as C,
  darkModeStore as D,
  interfaceAmoledTheme as E,
  themeColorStore as F,
  interfaceSearchSuggestionsStore as G,
  deArrowTitlesOnly as H,
  deArrowEnabledStore as I,
  deArrowInstanceStore as J,
  deArrowThumbnailInstanceStore as K,
  interfaceSearchHistoryEnabled as L,
  interfacePreviewVideoOnHoverStore as M,
  playerDefaultLanguage as N,
  playerAndroidBgPlayer as O,
  playerAndroidLockOrientation as P,
  playerAutoplayNextByDefaultStore as Q,
  sponsorBlockCategoriesStore as R,
  sponsorBlockStore as S,
  sponsorBlockDisplayToastStore as T,
  sponsorBlockTimelineStore as U,
  sponsorBlockUrlStore as V,
  showWarningStore as W,
  activePageStore as a,
  authStore as b,
  interfaceDisplayThumbnailAvatars as c,
  playerSavePlaybackPositionStore as d,
  interfaceForceCase as e,
  instanceStore as f,
  interfaceDefaultPage as g,
  returnYTDislikesInstanceStore as h,
  interfaceLowBandwidthMode as i,
  poTokenCacheStore as j,
  interfaceRegionStore as k,
  playerYouTubeJsAlways as l,
  playerYouTubeJsFallback as m,
  synciousInstanceStore as n,
  silenceSkipperStore as o,
  playerProxyVideosStore as p,
  playerAutoPlayStore as q,
  returnYtDislikesStore as r,
  syncPartyPeerStore as s,
  playerAlwaysLoopStore as t,
  synciousStore as u,
  playlistSettingsStore as v,
  syncPartyConnectionsStore as w,
  miniPlayerSrcStore as x,
  interfaceAutoExpandDesc as y,
  interfaceAutoExpandComments as z
};
